<?php

namespace Drupal\event_platform_details\Plugin\Menu;

use Drupal\Core\Menu\MenuLinkDefault;
use Drupal\Core\Menu\StaticMenuLinkOverridesInterface;
use Drupal\Core\Url;
use Drupal\config_pages\Entity\ConfigPages;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\pathauto\AliasCleaner;
use Drupal\taxonomy\Entity\Term;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Generates a dynamic link based on the current path.
 */
class EventMenuLinkBase extends MenuLinkDefault {

  /**
   * The text for the link created.
   *
   * @var string
   */
  protected $linkText = 'Example';

  /**
   * The URL suffix (last segment) for the link created.
   *
   * @var string
   */
  protected $suffix = 'example';

  /**
   * The URL prefix (first segment) for the links.
   *
   * @var string
   */
  protected $prefix = 'events';

  /**
   * The machine name of the config page to retrieve.
   *
   * @var string
   */
  protected $detailsPage = 'event_details';

  /**
   * The machine name of the config page field to retrieve.
   *
   * @var string
   */
  protected $extractField = 'field_current';

  /**
   * The config page onject.
   *
   * @var \Drupal\config_pages\Entity\ConfigPages|null
   */
  protected $configPage = NULL;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $pathAliasManager;

  /**
   * Pathauto alias cleaner.
   *
   * @var \Drupal\pathauto\AliasCleaner
   */
  protected $aliasCleaner;

  /**
   * Define class constructor.
   *
   * @param array $configuration
   *   Plugin configuration.
   * @param mixed $plugin_id
   *   Plugin id.
   * @param mixed $plugin_definition
   *   Plugin definition.
   * @param Drupal\Core\Menu\StaticMenuLinkOverridesInterface $static_override
   *   Static override.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The current request object.
   * @param \Drupal\path_alias\AliasManagerInterface $path_alias_manager
   *   Manager class for path aliases.
   * @param \Drupal\pathauto\AliasCleaner $alias_cleaner
   *   The request stack service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    StaticMenuLinkOverridesInterface $static_override,
    RequestStack $request_stack,
    AliasManagerInterface $path_alias_manager,
    AliasCleaner $alias_cleaner,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $static_override);
    $this->requestStack = $request_stack->getCurrentRequest();
    $this->pathAliasManager = $path_alias_manager;
    $this->aliasCleaner = $alias_cleaner;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('menu_link.static.overrides'),
      $container->get('request_stack'),
      $container->get('path_alias.manager'),
      $container->get('pathauto.alias_cleaner')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getTitle() {
    return $this->t($this->linkText);
  }

  /**
   * {@inheritdoc}
   */
  public function getUrlObject($title_attribute = TRUE) {
    // Get the current path and attempt to extract an event id.
    $path = $this->requestStack->getPathInfo();
    $path_token = '';
    $current_event_val = NULL;
    if (strlen($path) > 1) {
      $path_segments = explode('/', $path);
      if ($path_segments[1] === $this->prefix) {
        $term_path = '/' . $this->prefix . '/' . $path_segments[2];
        $path = $this->pathAliasManager->getPathByAlias($term_path);
        if (preg_match('/term\/(\d+)/', $path, $matches)) {
          $current_event_val = $matches[1];
          // @todo validate that term found is of the expected vocabulary.
          $path_token = $path_segments[2];
        }
      }
    }

    // Fall back to the site configured default event.
    if (!$path_token) {
      $configPage = ConfigPages::config($this->detailsPage);
      if (!$configPage) {
        return Url::fromUri('internal:/');
      }
      $current_event_val = $configPage->get($this->extractField)?->getValue();
      // Extract the value from within a nested array.
      while (is_array($current_event_val)) {
        $current_event_val = array_pop($current_event_val);
      }
      $this->configPage = $configPage;
      $current_event = Term::load($current_event_val);
      $event_title = $current_event->label();

      $path_token = $this->aliasCleaner->cleanString($event_title);
    }

    $url = Url::fromUri('internal:/' . $this->prefix . '/' . $path_token . '/' . $this->suffix);

    return $url;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    $contexts = ['url.path'];
    return $contexts;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    $tags = [];
    if ($this->configPage) {
      $tags[] = 'config_pages:' . $this->configPage->id();
    }
    return $tags;
  }

}
